/**
 *                           _    _   _ __    _ __
 *                          | |  | | | '_ \  | '_ \
 *                          | |  | | | |_) | | |_) |
 *                          | |  | | | .__/  | '_ )
 *                          | \__/ | | |     | |_) |
 *                          \______/ |_|     |_.__/
 *                          -------- _______ -------
 *
 *
 *  File:           main.c
 *  Description:    Automatic Guitar Tuner
 *  By:             ioan_florin.nitu
 *
 *   _______________________________________
 *  / Look for my W-USO ;)                  \
 *  \ (No, seriously, don't look...)        /
 *   ---------------------------------------
 *          \   ^__^
 *           \  (oo)\_______
 *              (__)\       )\/\
 *                  ||----w |
 *                  ||     ||
 *
 */

/** PROIECT
 *
 * main.c
 *
 * @author Nitu Ioan Florin Catalin
 * @group 333CC
 * @version 1.0
 * @since Rock 'n' Roll
 *
 * Copyright (c) 2019 PM
 *
 */

#pragma message("TUNER: Nitu Ioan-Florin-Catalin, 333CC, CTI, AC, UPB, 2019.\n")

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "led.h"
#include "lcd.h"
#include "btn.h"
#include "microphone.h"
#include "stepper.h"

#include "music.h"

#define FREQ_TOLERANCE 10.0f

#define TUNINGS 4
#define CUSTOM_TUNING 0
#define STANDARD_TUNING 1
#define DROP_D_TUNING 2
#define D_TUNING 3

#define BUFF_SIZE 20

char *tunings_menu[TUNINGS] = {
				"--->Customs<---",
				"E tuning       ",
				"Drop D tuning  ",
				"D tuning       "};

char *note;
float *freqs;
float freq = 0.0f;
int count_freq;

int max_index;
int index;
char **menu;

void TIMER1_init(void)
{
	/* set timer to one second, or one Hertz */
	TCCR1B |= (1 << CS12) | (1 << WGM12);
	TIMSK1 |= (1 << OCIE1A);
	OCR1A = 62500 - 1;
}

void TIMER1_free(void)
{
	TCCR1B = 0;
	TIMSK1 = 0;
	OCR1A = 0;
}

ISR(TIMER1_COMPA_vect)
{
	TIMER1_free();
	char buff[BUFF_SIZE];
	float f = count_freq;
	float df = freq - f;

	LCD_clear();
	LCD_printAt(LCD_FIRST_LINE_BEGIN, note);
	sprintf(buff, "%d", count_freq);
	LCD_printAt(LCD_SECOND_LINE_BEGIN, buff);
	if (fabsf(df) < FREQ_TOLERANCE) {
		/* note frequency is in tolerance interval */
		note = NULL;
		freq = 0.0f;
		_delay_ms(1000);
		LCD_clear();
		LCD_printAt(LCD_FIRST_LINE_BEGIN, note);
		LCD_printAt(LCD_SECOND_LINE_BEGIN, "OK");
		count_freq = 0;
	} else {
		/* tune note */
		if (f > FREQ_TOLERANCE)
			STEPPER_rotation((df > 0 ? 45 : -45));
		count_freq = 0;
		TIMER1_init();
	}
}

ISR(PCINT0_vect)
{
	/* count changes of sign in oscillation */
	if ((MICROPHONE_PIN & (1 << MICROPHONE_P)) == 0)
		count_freq++;
}

ISR(PCINT1_vect)
{
	static int old_BTN_PIN = -1;
	static float tmp_freq = 0.0f;

	if (old_BTN_PIN == -1)
		old_BTN_PIN = BTN_PIN;
	if ((BTN_PIN & (1 << BTN_DOWN_P)) == 0 &&
		(old_BTN_PIN & (1 << BTN_DOWN_P)) != 0) {
		LCD_clear();
		LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
		index = (index + 1) % max_index;
		LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
	}
	if ((BTN_PIN & (1 << BTN_UP_P)) == 0 &&
		(old_BTN_PIN & (1 << BTN_UP_P)) != 0) {
		LCD_clear();
		LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
		index = (max_index + index - 1) % max_index;
		LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
	}
	if ((BTN_PIN & (1 << BTN_OK_P)) == 0 &&
		(old_BTN_PIN & (1 << BTN_OK_P)) != 0) {
		LCD_clear();
		/* select next menu */
		if (menu == tunings_menu) {
			max_index = STRINGS;
			switch (index) {
			case STANDARD_TUNING:
				menu = standard_names;
				freqs = standard_freqs;
				break;
			case DROP_D_TUNING:
				menu = drop_d_names;
				freqs = drop_d_freqs;
				break;
			case D_TUNING:
				menu = d_names;
				freqs = d_freqs;
				break;
			default:
				menu = note_names;
				max_index = NOTES;
				break;
			}
			index = max_index - 1;
			LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
			index = (index + 1) % max_index;
			LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
		} else {
			if (menu == note_names) {
				/* select custom note and pitch */
				tmp_freq = note_freqs[index];
				menu = note_pitch;
				max_index = PITCHES;
				for (int i = 0; i < max_index; i++) {
					strncpy(menu[i], note_names[index],
						strlen(note_names[index]));
				}
				index = max_index - 1;
				LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
				index = (index + 1) % max_index;
				LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
			} else {
				/* start tuning */
				_delay_ms(1000);
				note = menu[index];
				if (menu == note_pitch)
					freq = get_freq(tmp_freq, index);
				else
					freq = freqs[index];
				LCD_printAt(LCD_FIRST_LINE_BEGIN, note);
				count_freq = 0;
				TIMER1_init();
			}
		}
	}
	if ((BTN_PIN & (1 << BTN_BACK_P)) == 0 &&
		(old_BTN_PIN & (1 << BTN_BACK_P)) != 0) {
		/* select prev menu */
		LCD_clear();
		if (note == NULL && menu != tunings_menu) {
			menu = tunings_menu;
			max_index = TUNINGS;
			index = max_index - 1;
			LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
			index = (index + 1) % max_index;
			LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
		} else {
			TIMER1_free();
			note = NULL;
			freq = 0.0f;
			LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
			index = (index + 1) % max_index;
			LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);
		}
	}
	old_BTN_PIN = BTN_PIN;
	_delay_ms(1);
}

void init(void)
{
	LED_init();
	BTN_init();
	LCD_init();
	MICROPHONE_init();
	STEPPER_init();
}

int main(void)
{
	init();

	LCD_printAt(LCD_FIRST_LINE_BEGIN, "Automatic Guitar");
	LCD_printAt(LCD_SECOND_LINE_BEGIN, "Tuner           ");
	_delay_ms(1000);

	LED_on();

	sei();

	menu = tunings_menu;
	max_index = TUNINGS;
	index = max_index - 1;

	LCD_clear();
	LCD_printAt(LCD_FIRST_LINE_BEGIN, menu[index]);
	index = (index + 1) % max_index;
	LCD_printAt(LCD_SECOND_LINE_BEGIN, menu[index]);

	while (1)
		;

	return 0;
}

